<?php
# This file is part of the Savane project
# <http://gna.org/projects/savane/>
#
# $Id: groups.php,v 1.20 2005/08/03 19:14:04 beuc Exp $
#
#  Copyright 2003-2005 (c) Frederik Orellana <frederik.orellana--cern.ch>
#                          Mathieu Roy <yeupou--gnu.org>
#
# The Savane project is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# The Savane project is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with the Savane project; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


require "../include/pre.php";
require_directory("search");
require_directory("trackers");

# ###### get global user and group vars

$res_user = db_query("SELECT * FROM user WHERE user_id=" . user_getid());
$row_user = db_fetch_array($res_user);
$sys_pending_user_permissions = $GLOBALS['sys_pending_user_permissions'];
$register_error=0;


$sql = "SELECT groups.group_name,"
. "groups.group_id,"
. "groups.unix_group_name,"
. "groups.status,"
. "user_group.admin_flags, "
. "group_history.date "
. "FROM groups,user_group,group_history "
. "WHERE groups.group_id=user_group.group_id "
. "AND user_group.user_id='".user_getid()."' "
. "AND groups.status='A' "
. "AND (group_history.field_name='Added User' OR group_history.field_name='Approved User' OR user_group.admin_flags='P')"
. "AND group_history.group_id=user_group.group_id "
. "AND group_history.old_value='".user_getname()."' "
. "GROUP BY groups.unix_group_name "
. "ORDER BY groups.unix_group_name";

$result = db_query($sql);
$rows = db_numrows($result);

# Alternative sql that do not use group_history, just in case this history
# would be flawed (history usage has been inconsistent over Savane history)
$sql_without_history = "SELECT groups.group_name,"
. "groups.group_id,"
. "groups.unix_group_name,"
. "groups.status,"
. "user_group.admin_flags "
. "FROM groups,user_group "
. "WHERE groups.group_id=user_group.group_id "
. "AND user_group.user_id='".user_getid()."' "
. "AND groups.status='A' "
. "GROUP BY groups.unix_group_name "
. "ORDER BY groups.unix_group_name";

$result_without_history = db_query($sql_without_history);
$rows_without_history = db_numrows($result_without_history);

if ($rows_without_history != $rows)
{
  # If number of rows differ, assume that history is flawed. Print a
  # feedback incitating to fix the installation and override flawed result
  fb(_("Groups history appears to be flawed. This is a site installation problem. Please report the incident to administrators, asking them to get in touch with their Savane supplier."), 1);
  $result = $result_without_history;
  $rows = $rows_without_history;
}



# ###### function send_pending_user_email($group_id, $user_id, $user_message)
# ###### sends an email to group admins when a user joins group

function send_pending_user_email($group_id, $user_id, $user_message)
{
	$GLOBALS[sys_debug_where] = __FILE__.':'.__LINE__.':send_pending_user_email($group_id, $user_id, $user_message) ';

  $res_grp = db_query("SELECT * FROM groups WHERE group_id='$group_id'");

  if (db_numrows($res_grp) < 1)
    {
      return 0;
    }

  $row_grp = db_fetch_array($res_grp);

  $res_admins = db_query("SELECT user.user_name FROM user,user_group WHERE "
			 . "user.user_id=user_group.user_id AND user_group.group_id='$group_id' AND "
			 . "user_group.admin_flags='A'");

  if (db_numrows($res_admins) < 1)
    {
      return 0;
    }

  # send one email per admin, in one command line coma separated
  $admin_list = '';
  while ($row_admins = db_fetch_array($res_admins))
    {
      $admin_list .= ($admin_list ? ',':'').$row_admins['user_name'];
    }

  $message = approval_user_gen_email($row_grp['group_name'],
				     $row_grp['unix_group_name'],
				     $group_id,
				     user_getname($user_id),
				     user_getrealname($user_id),
				     user_getemail($user_id),
				     $user_message);

  sendmail_mail(user_getname(),
		$admin_list,
		sprintf(_("Membership request for group %s"), $row_grp['group_name']),
		$message,
		$row_grp['unix_group_name'],
		"usermanagement");
}


# ###### function update_tbl()
# ###### updates tables if a post

function update_tbl()
{
  $GLOBALS[sys_debug_where] = __FILE__.':'.__LINE__.':update_tbl()	';
  $ret_val = 1;

  global $register_error;
  global $row_user;
  global $sys_pending_user_permissions;
  global $form_id;

  utils_get_content("my/request_for_inclusion");

  $result_upd = db_query("SELECT group_id FROM groups WHERE status='A' AND
            is_public='1' ORDER BY group_id");

  if ($GLOBALS["update"])
    {

      # Check for duplicates
      if (!form_check($form_id))
	{ return 0; }

      while ($val = db_fetch_array($result_upd))
	{
	  if($GLOBALS["form_groups_$val[group_id]"])
	    {
               # If not in group, add user with admin_flag "P"
               # (not very sensible, but this way we avoid changing
	       # the table layout)
	      if(!member_check_pending($row_user[user_id], $val[group_id]))
		{
		  if(!$GLOBALS["form_message"])
		    {
		      fb(_("Error: When joining you must provide a message for the administrator, a short explanation of why you want to join this/these project(s)."), 1);
		      $ret_val = 0;
		    }
		  else
		    {
		      if(member_add($row_user[user_id], $val[group_id], 'P'))
		      {
			send_pending_user_email($val[group_id], $row_user[user_id], $GLOBALS["form_message"]);
			form_clean($form_id);
		      }
		    }
		}
	      else
		{
		  fb(_("Request for inclusion already registered"),1);
		}
	    }
	}
      return $ret_val;
    }
  return 0;
}

# ###### first check if post, update requests

update_tbl();

# Some action to be done ?
if ($func)
{
  if ($func == "delwatchee")
    {
# Stop watching another user
      $result_upd = trackers_data_delete_watchees(user_getid(),$watchee_id,$group_id);
      if (!$result_upd)
	{
	  fb(_("Unable to remove user from the watched users list, probably a broken URL"));
	}

    }

  if ($func == "addwatchee")
    {
# Stop watching another user
      $result_upd = trackers_data_add_watchees(user_getid(),$watchee_id,$group_id);
      if (!$result_upd)
	{
	  fb(_("Unable to add user in the watched users list, probably a broken URL"));
	}

    }
}


# page header
site_user_header(array('toptab'=>'mygroups'));

# we get site-specific content
utils_get_content("my/groups");
print "<br /><br />\n\n";


################ RIGHT PART ###########################

print html_splitpage(1);  # Watching other users.

print $HTML->box_top(_("Watched Partners"));

$result_w = trackers_data_get_watchees(user_getid());
$rows_w=db_numrows($result_w);

if (!$result_w || $rows_w < 1)
    {
      print _("I am not watching any partners.");
      print '<p>'._("Watching a partner (receiving a copy of all the notification sent to him) permits to be his backup when he is away from the office, or to do QA to all his/her activities on a project.");
      print '</p><p>'._("To watch someone, click 'Watch Partner' in any project memberlist page (membership to that project is required, indeed).");
      print '<br />&nbsp;';
      print db_error();
    }
else
    {
      print '<table>';
      for ($i=0; $i<$rows_w; $i++)
	{
	  print '<tr class="'.utils_get_alt_row_color($i).'"><td width="99%"><font class="bold">'.
	    utils_user_link(user_getname(db_result($result_w, $i, 'watchee_id')), user_getrealname(db_result($result_w, $i, 'watchee_id'))).
	    '</font> <font class="smaller">['.group_getname(db_result($result_w, $i, 'group_id')).']'.
	    '</font>';

	  print '</td>'.
	    '<td><a href="'.$_SERVER['PHP_SELF'].'?func=delwatchee&amp;group_id='.db_result($result_w,$i,'group_id').'&amp;watchee_id='.db_result($result_w, $i, 'watchee_id').
	    '" onClick="return confirm(\''._("Stop watching this user?").'\')">'.
	    '<img src="'.$GLOBALS['sys_home'].'images/'.SV_THEME.'.theme/trash.png" border="0" alt="'._("Stop watching this user?").'" /></a></td></tr>';
	}
      print '</table>';

    }

  $result_w = trackers_data_get_watchers(user_getid());
  $arr_watchers = array();
  while ($row_watcher = db_fetch_array($result_w))
    {
      $watchers .= utils_user_link(user_getname($row_watcher['user_id']), user_getrealname($row_watcher['user_id'])).'</font> <font class="smaller">['.group_getname($row_watcher['group_id']).']</font>, ';
    }

  if ($watchers)
    {
      $watchers = substr($watchers,0,-2); # remove extra comma at the end
      $watchers .= ".";

      print '<p>';
      printf (_("My own notifications are currently watched by: %s"),$watchers);
      print '</p>';
    }
  else
    {
      print '<p>'._("Nobody is currently watching my own notifications.").'</p>';
    }

print $HTML->box_bottom();

print "<br />\n";

print $HTML->box_top(_("Request for Inclusion"),'',1);

print '<div class="boxitem">'._("Search a Group")."\n";
  print '<p class="text">';
print _("If there is a project - or several - you would like to be member of, to be able to fully contribute, it is possible to search his/their name(s) in the whole group database with the following search tool. A list of groups will be generated, depending on the word(s) typed in this form.")."\n";
print '</p>';

print '
	<form action="'.$PHP_SELF.'#searchgroup" method="post">
	<input type="hidden" name="action" value="searchgroup" />
        <input type="text" size="35" name="words" value="'.$words.'" /><br />
	<br /><br />
	<input type="submit" name="Submit" value="'
	._("Search Group(s)").'" />
	</form>

</div><!-- end boxitem -->';


if ($words)
{
  # Avoid to big search by asking for more than 3 characters.
  if (strlen($words) > 3)
    {
      $result_search = search_run($words, "soft", 0);
    }
  else
    { $result_search = 0; }

  print '<div class="boxitemalt"><a name="searchgroup"></a>'._("Requesting Inclusion");
  print '<p class="text">';
  print _("Below is the result of the research in the groups database.");
  print '</p>';

  if (db_numrows($result_search) < 1)
    {
      print '<br /><font class="warn">'._("None found. Please note that only search words of more than three characters are valid.").'</font><br />';
    }
  else
    {
      # We do not put pointer to group page along with checkbox,
      # to avoid creating any confusion (for instance, should I check the
      # box or click on the link?).
      # This tool is to search groups for inclusion, not to look around
      # to get information about groups.
      print '<p class="text">';
      print _("To request inclusion in one or several groups, check the correspondants boxes, write a meaningful message for the project administrator, that will approve or disapprove the request, and submit the form.");

      print '</p>'.form_header($PHP_SELF);

      while ($val = db_fetch_array($result_search))
	{
	  if (!user_is_group_member($row_user[user_id], $val[group_id]))
	    {
	      print '<input type="checkbox" name="form_groups_'.$val[group_id].'" /> ';
	      print $val[group_name];
	      print '<br />';
	    }
	}

      print '<br />'._("Comments (required):").'<br />
     <textarea name="form_message" cols="40" rows="7"></textarea><br /><br />
     <input type="submit" name="update" value="';
      print _("Request Inclusion").'" /></form>';
    }
  print '</div><!-- end boxitemalt -->';
}

print $HTML->box_bottom(1);


print html_splitpage(2);

################ LEFT PART ###########################


if (!$result || $rows < 1)
{

  print $HTML->box_top(_("My Groups"),'',1);
  print _("You're not a member of any public projects");
  print $HTML->box_bottom(1);

}
else
{

  /*
     Projects administrated by the user
  */

  print $HTML->box_top(_("Groups I'm Administrator of"),'',1);

  $j = 1;
  unset($content);
  for ($i=0; $i<$rows; $i++)
    {
      if (db_result($result,$i,'admin_flags') == 'A')
	{
	  $content .= '<li class="'.utils_get_alt_row_color($j).'">';
	  $content .= '<span class="trash"><a href="../my/quitproject.php?quitting_group_id='. db_result($result,$i,'group_id').
	    '">'.
	   '<img src="'.$GLOBALS['sys_home'].'images/'.SV_THEME.'.theme/trash.png" alt="'._("Quit this project?").'" /></a><br /></span>';

	  $content .= '<a href="'.$GLOBALS['sys_home'].'projects/'. db_result($result,$i,'unix_group_name') .'/">'.db_result($result,$i,'group_name').'</a><br />';
	  if (db_result($result,$i,'date'))
	    {
	      # If the group history is flawed (site install problem), the
	      # date may be unavailable
	      $content .= '<font class="smaller">'.sprintf(_("Member since %s"), strftime('%B %Y',db_result($result,$i,'date'))).'</font>';
	    }
	  $content .= '</li>';
	  $exists=1;
	  $j++;
	}
    }
  if (!$exists)
    {
      print _("I am not administrator of any projects");
    }
  else
    {
      print '<ul class="boxli">'.$content.'</ul>';
    }
  unset($exists);

  print $HTML->box_bottom(1);

  print "<br />\n";

  /*
     Projects the user is member of
  */

  print $HTML->box_top(_("Groups I'm Contributor of"),'',1);

  $j = 1;
  unset($content);
  for ($i=0; $i<$rows; $i++)
    {
      if (db_result($result,$i,'admin_flags') == '')
	{
	  $content .= '<li class="'.utils_get_alt_row_color($j).'">';
	  $content .= '<span class="trash"><a href="../my/quitproject.php?quitting_group_id='. db_result($result,$i,'group_id').
	    '">'.
	   '<img src="'.$GLOBALS['sys_home'].'images/'.SV_THEME.'.theme/trash.png" alt="'._("Quit this project?").'" /></a></span>';

	  $content .= '<a href="'.$GLOBALS['sys_home'].'projects/'. db_result($result,$i,'unix_group_name') .'/">'.db_result($result,$i,'group_name').'</a><br />';
	  if (db_result($result,$i,'date'))
	    {
	      # If the group history is flawed (site install problem), the
	      # date may be unavailable
	      $content .= '<font class="smaller">'.sprintf(_("Member since %s"), strftime('%B %Y',db_result($result,$i,'date'))).'</font>';
	    }
	  $content .= '</li>';
	  $exists=1;
	  $j++;
	}
    }

  if (!$exists)
    {
      print _("I am not contributor member of any projects");
    }
  else
    {
      print '<ul class="boxli">'.$content.'</ul>';
    }
  unset($exists);

  print $HTML->box_bottom(1);


print "<br />\n";

/*
     Projects the user requested to be member of
*/

print $HTML->box_top(_("Request for Inclusion Waiting For Approval"),'',1);

unset($content);

for ($i=0; $i<$rows; $i++)
{
  if (db_result($result,$i,'admin_flags') == 'P')
    {
      $content .= '<li class="'.utils_get_alt_row_color($j).'">';
      $content .= '<span class="trash"><a href="../my/quitproject.php?quitting_group_id='. db_result($result,$i,'group_id').'&amp;pending=1">'.
	   '<img src="'.$GLOBALS['sys_home'].'images/'.SV_THEME.'.theme/trash.png" alt="'._("Discard this request?").'" /></a></span>';

      $content .= '<a href="'.$GLOBALS['sys_home'].'projects/'. db_result($result,$i,'unix_group_name') .'/">'.db_result($result,$i,'group_name').'</a><br />&nbsp;</li>';
      $exists=1;

    }
}

if (!$exists)
{
  print _("None found");
}
else
{
  print '<ul class="boxli">'.$content.'</ul>';
}
unset($exists);

print $HTML->box_bottom(1);

}

print html_splitpage(3);



$HTML->footer(array());

?>